// import 'macros.c' as macros
#pragma once

/* Generated by xpdt
 * xpdt is written by Gianni Tedesco
 * https://github.com/giannitedesco/xpdt
 */

/*# for hdr in headers #*/
#include /*{hdr.path_spec}*/
/*# endfor #*/

/*{-macros.discr_enum(namespace)}*/
/*#- for struct in namespace #*/

void /*{macros.entry_name(namespace, struct)}*/(/*{macros.cdecls(struct)}*/);
/*#- endfor #*/
