// import 'macros.c' as macros
/* Generated by xpdt
 * xpdt is written by Gianni Tedesco
 * https://github.com/giannitedesco/xpdt
 */

/*# for hdr in headers #*/
#include /*{hdr.path_spec}*/
/*# endfor #*/

extern struct xostream /*{macros.stream(namespace)}*/;
/*#- for struct in namespace #*/


void /*{macros.entry_name(namespace, struct)}*/(/*{macros.cdecls(struct)}*/)
{
/*# if struct.has_discriminant #*/
	/*{macros.write_wrapped(struct)}*/(&/*{macros.stream(namespace)}*/,
		/*{macros.memb_names(struct)}*/);
/*# else #*/
	/*{macros.write_fixed(struct)}*/(&/*{macros.stream(namespace)}*/,
		/*{macros.memb_names(struct)}*/);
/*# endif #*/
}
/*#- endfor #*/
